#!/usr/bin/env python
# coding: utf-8

# In[1]:


import pandas as pd
import numpy as np
from numpy import interp
import scipy.stats

import joblib

import sklearn.preprocessing
import sklearn.pipeline
import sklearn.impute
import sklearn.svm
import sklearn.model_selection
import sklearn.metrics

import matplotlib.pyplot as plt
import seaborn as sns


# # Load data

# In[2]:


# load data
data_stress = pd.read_csv("data_stress.csv", header=[0,1], index_col=0)


# # Train model

# In[3]:


estimator = sklearn.pipeline.make_pipeline(
    sklearn.impute.SimpleImputer(strategy="median"),
    sklearn.preprocessing.StandardScaler(),
    sklearn.svm.SVC()
)


# In[4]:


n_repeat = 100
n_jobs = 10

def predict():
    return sklearn.model_selection.cross_val_predict(
        estimator, 
        data_stress.features.values, 
        data_stress.meta.preterm.values,
        cv=sklearn.model_selection.KFold(n_splits=10, shuffle=True),
        method="decision_function")

results = joblib.Parallel(n_jobs=n_jobs)(joblib.delayed(predict)() for _ in range(n_repeat))


# In[5]:


auc_scores = []
pvalues = []
for r in results:
    auc = sklearn.metrics.roc_auc_score(data_stress.meta.preterm.values, r)
    auc_scores.append(auc)
    
    values_preterm = r[data_stress.meta.preterm.values == True]
    values_term = r[data_stress.meta.preterm.values == False]
    s, p = scipy.stats.ranksums(values_term, values_preterm)
    pvalues.append(p)


# # Analyze results

# ## Basic stats

# In[6]:


print(f"ROC AUC: {np.mean(auc_scores):.02f} +/- {np.std(auc_scores):.02f}")
print(f"p-value (Wilcoxon Rank-Sum): {np.mean(pvalues):.02e} +/- {np.std(pvalues):.02e}")


# In[7]:


# print(f"ROC AUC: {np.median(auc_scores):.02f} +/- {np.std(auc_scores):.02f}")
# print(f"p-value (Wilcoxon Rank-Sum): {np.median(pvalues):.02e} +/- {np.std(pvalues):.02e}")


# ## ROC

# In[8]:


fig, ax = plt.subplots(1,1)

color = "blue"

tprs = []
aucs = []
mean_fpr = np.linspace(0, 1, 100)

y = data_stress.meta.preterm.values
for i, r in enumerate(results):

    values_ = r

    # Compute ROC curve and area the curve
    fpr, tpr, thresholds = sklearn.metrics.roc_curve(y, values_)
    tprs.append(np.interp(mean_fpr, fpr, tpr))
    tprs[-1][0] = 0.0
    roc_auc = sklearn.metrics.auc(fpr, tpr)
    aucs.append(roc_auc)

mean_tpr = np.mean(tprs, axis=0)
mean_tpr[-1] = 1.0
mean_auc = sklearn.metrics.auc(mean_fpr, mean_tpr)
std_auc = np.std(aucs)
ax.plot(
    mean_fpr, mean_tpr, 
    color=color, label=r'Mean ROC (AUC = %0.2f $\pm$ %0.2f)' % (mean_auc, std_auc), 
    lw=2, alpha=.8)

std_tpr = np.std(tprs, axis=0)
tprs_upper = np.minimum(mean_tpr + std_tpr, 1)
tprs_lower = np.maximum(mean_tpr - std_tpr, 0)

ax.fill_between(
    mean_fpr, tprs_lower, tprs_upper, 
    color=color, alpha=.1, label=r'$\pm$ 1 std. dev.')

ax.plot(
    [0, 1], [0, 1], 
    linestyle='--', lw=2, color='grey', label='Chance', alpha=.8)

ax.set_xlim([-0.05, 1.05])
ax.set_ylim([-0.05, 1.05])
ax.set_xlabel('False Positive Rate')
ax.set_ylabel('True Positive Rate')
ax.set_title('Receiver operating characteristic')
ax.legend(bbox_to_anchor=(1.04, 1), loc="upper left")

fig.savefig("roc.png", bbox_inches='tight')


# ## Histogram

# In[9]:


mean_values = np.mean(results, axis=0)

fig, ax = plt.subplots(1,1)
sns.boxplot(x=data_stress.meta.preterm.values, y=mean_values, ax=ax)
ax.set_xticklabels(["Term", "Preterm"]);
ax.set_ylabel("mean decision function value")

fig.savefig("hist.png")


# In[ ]:




